package com.sun.xml.internal.ws.policy.sourcemodel;

import com.sun.xml.internal.ws.policy.NestedPolicy;
import com.sun.xml.internal.ws.policy.PolicyException;
import java.util.Iterator;
import com.sun.xml.internal.ws.policy.privateutil.LocalizationMessages;
import com.sun.xml.internal.ws.policy.PolicyAssertion;
import com.sun.xml.internal.ws.policy.AssertionSet;
import com.sun.xml.internal.ws.policy.Policy;
import com.sun.xml.internal.ws.policy.privateutil.PolicyLogger;

class NormalizedModelGenerator extends PolicyModelGenerator
{
    private static final PolicyLogger LOGGER;
    private final PolicyModelGenerator.PolicySourceModelCreator sourceModelCreator;
    
    NormalizedModelGenerator(PolicyModelGenerator.PolicySourceModelCreator sourceModelCreator) {
        this.sourceModelCreator = sourceModelCreator;
    }
    
    @Override
    public PolicySourceModel translate(Policy policy) throws PolicyException {
        NormalizedModelGenerator.LOGGER.entering(new Object[] { policy });
        PolicySourceModel model = null;
        if (policy == null) {
            NormalizedModelGenerator.LOGGER.fine(LocalizationMessages.WSP_0047_POLICY_IS_NULL_RETURNING());
        }
        else {
            model = this.sourceModelCreator.create(policy);
            ModelNode rootNode = model.getRootNode();
            ModelNode exactlyOneNode = rootNode.createChildExactlyOneNode();
            for (AssertionSet set : policy) {
                ModelNode alternativeNode = exactlyOneNode.createChildAllNode();
                for (PolicyAssertion assertion : set) {
                    AssertionData data = AssertionData.createAssertionData(assertion.getName(), assertion.getValue(), assertion.getAttributes(), assertion.isOptional(), assertion.isIgnorable());
                    ModelNode assertionNode = alternativeNode.createChildAssertionNode(data);
                    if (assertion.hasNestedPolicy()) {
                        this.translate(assertionNode, assertion.getNestedPolicy());
                    }
                    if (assertion.hasParameters()) {
                        super.translate(assertionNode, assertion.getParametersIterator());
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        NormalizedModelGenerator.LOGGER.exiting(model);
        return model;
    }
    
    @Override
    protected ModelNode translate(ModelNode parentAssertion, NestedPolicy policy) {
        ModelNode nestedPolicyRoot = parentAssertion.createChildPolicyNode();
        ModelNode exactlyOneNode = nestedPolicyRoot.createChildExactlyOneNode();
        AssertionSet set = policy.getAssertionSet();
        ModelNode alternativeNode = exactlyOneNode.createChildAllNode();
        super.translate(alternativeNode, set);
        return nestedPolicyRoot;
    }
    
    static {
        LOGGER = PolicyLogger.getLogger(NormalizedModelGenerator.class);
    }
}
